@@ -29,6 +29,7 @@ should have been included in this package.
 """
 
 import os
+import sys
 import pprint
 import base64
 import posixpath
@@ -49,19 +50,12 @@ except ImportError:
 from . import auth, config, ical, log, rights, storage, xmlutils
 
 
-VERSION = "0.8"
+VERSION = "0.9"
 
 # Standard "not allowed" response that is returned when an authenticated user
 # tries to access information they don't have rights to
 NOT_ALLOWED = (client.FORBIDDEN, {}, None)
 
-# Standard "authenticate" response that is returned when a user tries to access
-# non-public information w/o submitting proper authentication credentials
-WRONG_CREDENTIALS = (
-    client.UNAUTHORIZED,
-    {"WWW-Authenticate": "Basic realm=\"%s\"" % config.get("server", "realm")},
-    None)
-
 
 class HTTPServer(wsgiref.simple_server.WSGIServer, object):
     """HTTP server."""
@@ -96,16 +90,22 @@ class HTTPSServer(HTTPServer):
             try:
                 open(filename, "r").close()
             except IOError as exception:
-                log.LOGGER.warn(
+                log.LOGGER.warning(
                     "Error while reading SSL %s %r: %s" % (
                         name, filename, exception))
 
-        self.socket = ssl.wrap_socket(
-            self.socket,
+        ssl_kwargs = dict(
             server_side=True,
             certfile=config.get("server", "certificate"),
             keyfile=config.get("server", "key"),
-            ssl_version=ssl.PROTOCOL_SSLv23)
+            ssl_version=getattr(ssl, config.get("server", "protocol"),
+                                ssl.PROTOCOL_SSLv23)
+        )
+        # add ciphers argument only if supported (Python 2.7+)
+        if sys.version_info >= (2, 7):
+            ssl_kwargs["ciphers"] = config.get("server", "ciphers") or None
+
+        self.socket = ssl.wrap_socket(self.socket, **ssl_kwargs)
 
         self.server_bind()
         self.server_activate()
@@ -131,8 +131,8 @@ class Application(object):
         """Initialize application."""
         super(Application, self).__init__()
         auth.load()
-        rights.load()
         storage.load()
+        rights.load()
         self.encoding = config.get("encoding", "request")
         if config.getboolean("logging", "full_environment"):
             self.headers_log = lambda environ: environ
@@ -190,7 +190,7 @@ class Application(object):
 
         for item in items:
             if isinstance(item, ical.Collection):
-                if rights.read_authorized(user, item):
+                if rights.authorized(user, item, "r"):
                     log.LOGGER.debug(
                         "%s has read access to collection %s" %
                         (user or "Anonymous", item.url or "/"))
@@ -202,7 +202,7 @@ class Application(object):
                         (user or "Anonymous", item.url or "/"))
                     read_last_collection_allowed = False
 
-                if rights.write_authorized(user, item):
+                if rights.authorized(user, item, "w"):
                     log.LOGGER.debug(
                         "%s has write access to collection %s" %
                         (user or "Anonymous", item.url or "/"))
@@ -252,6 +252,9 @@ class Application(object):
             environ["PATH_INFO"] = self.sanitize_uri(
                 "/%s" % environ["PATH_INFO"][len(base_prefix):])
             log.LOGGER.debug("Sanitized path: %s", environ["PATH_INFO"])
+        elif config.get("server", "can_skip_base_prefix"):
+            log.LOGGER.debug(
+                "Skipped already sanitized path: %s", environ["PATH_INFO"])
         else:
             # Request path not starting with base_prefix, not allowed
             log.LOGGER.debug(
@@ -269,9 +272,6 @@ class Application(object):
 
         path = environ["PATH_INFO"]
 
-        # Find collection(s)
-        items = ical.Collection.from_path(path, environ.get("HTTP_DEPTH", "0"))
-
         # Get function corresponding to method
         function = getattr(self, environ["REQUEST_METHOD"].lower())
 
@@ -279,33 +279,44 @@ class Application(object):
         authorization = environ.get("HTTP_AUTHORIZATION", None)
 
         if authorization:
-            authorization = \
-                authorization.lstrip("Basic").strip().encode("ascii")
-            user, password = self.decode(
-                base64.b64decode(authorization), environ).split(":", 1)
+            authorization = authorization.lstrip("Basic").strip()
+            user, password = self.decode(base64.b64decode(
+                authorization.encode("ascii")), environ).split(":", 1)
         else:
-            user = password = None
+            user = environ.get("REMOTE_USER")
+            password = None
 
-        if not items or function == self.options or \
-                auth.is_authenticated(user, password):
+        is_authenticated = auth.is_authenticated(user, password)
+        is_valid_user = is_authenticated or not user
 
+        if is_valid_user:
+            items = ical.Collection.from_path(
+                path, environ.get("HTTP_DEPTH", "0"))
             read_allowed_items, write_allowed_items = \
                 self.collect_allowed_items(items, user)
-
-            if read_allowed_items or write_allowed_items or \
-                    function == self.options or not items:
-                # Collections found, or OPTIONS request, or no items at all
-                status, headers, answer = function(
-                    environ, read_allowed_items, write_allowed_items, content,
-                    user)
-            else:
-                # Good user but has no rights to any of the given collections
-                status, headers, answer = NOT_ALLOWED
         else:
+            read_allowed_items, write_allowed_items = None, None
+
+        if is_valid_user and (
+                (read_allowed_items or write_allowed_items) or
+                (is_authenticated and function == self.propfind) or
+                function == self.options):
+            status, headers, answer = function(
+                environ, read_allowed_items, write_allowed_items, content,
+                user)
+        else:
+            status, headers, answer = NOT_ALLOWED
+
+        if ((status, headers, answer) == NOT_ALLOWED and
+                not auth.is_authenticated(user, password) and
+                config.get("auth", "type") != "None"):
             # Unknown or unauthorized user
-            log.LOGGER.info(
-                "%s refused" % (user or "Anonymous user"))
-            status, headers, answer = WRONG_CREDENTIALS
+            log.LOGGER.info("%s refused" % (user or "Anonymous user"))
+            status = client.UNAUTHORIZED
+            headers = {
+                "WWW-Authenticate":
+                "Basic realm=\"%s\"" % config.get("server", "realm")}
+            answer = None
 
         # Set content length
         if answer:
@@ -332,7 +343,7 @@ class Application(object):
                user):
         """Manage DELETE request."""
         if not len(write_collections):
-            return client.PRECONDITION_FAILED, {}, None
+            return NOT_ALLOWED
 
         collection = write_collections[0]
 
@@ -548,7 +559,7 @@ class Application(object):
             xmlutils.put(environ["PATH_INFO"], content, collection)
             status = client.CREATED
             # Try to return the etag in the header.
-            # If the added item does't have the same name as the one given
+            # If the added item doesn't have the same name as the one given
             # by the client, then there's no obvious way to generate an
             # etag, we can safely ignore it.
             new_item = collection.get_item(item_name)
