@@ -45,16 +45,18 @@ INITIAL_CONFIG = {
         "ssl": "False",
         "certificate": "/etc/radicale/server.crt",
         "key": "/etc/radicale/private/server.key",
+        "protocol": "PROTOCOL_SSLv23",
+        "ciphers": "",
         "dns_lookup": "True",
         "base_prefix": "/",
+        "can_skip_base_prefix": "False",
         "realm": "Radicale - Password Required"},
     "encoding": {
         "request": "utf-8",
         "stock": "utf-8"},
     "auth": {
         "type": "None",
-        "public_users": "public",
-        "private_users": "private",
+        "custom_handler": "",
         "htpasswd_filename": "/etc/radicale/users",
         "htpasswd_encryption": "crypt",
         "imap_hostname": "localhost",
@@ -72,11 +74,15 @@ INITIAL_CONFIG = {
         "http_url": "",
         "http_user_parameter": "",
         "http_password_parameter": ""},
+    "git": {
+        "committer": "Radicale <radicale@example.com>"},
     "rights": {
         "type": "None",
-        "file": ""},
+        "custom_handler": "",
+        "file": "~/.config/radicale/rights"},
     "storage": {
         "type": "filesystem",
+        "custom_handler": "",
         "filesystem_folder": "/var/db/radicale/calendars",
         "database_url": ""},
     "logging": {
