@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of Radicale Server - Calendar Server
-# Copyright © 2008-2012 Guillaume Ayoub
 # Copyright © 2008 Nicolas Kandel
 # Copyright © 2008 Pascal Halter
+# Copyright © 2008-2013 Guillaume Ayoub
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,13 +32,24 @@ try:
 except ImportError:
     # Python 2.6 has no OrderedDict, use a dict instead
     OrderedDict = dict  # pylint: disable=C0103
+
+# Manage Python2/3 different modules
+# pylint: disable=F0401,E0611
+try:
+    from urllib.parse import unquote, urlparse
+except ImportError:
+    from urllib import unquote
+    from urlparse import urlparse
+# pylint: enable=F0401,E0611
+
 import re
 import xml.etree.ElementTree as ET
 
-from radicale import client, config, ical
+from . import client, config, ical
 
 
 NAMESPACES = {
+    "A": "http://apple.com/ns/ical/",
     "C": "urn:ietf:params:xml:ns:caldav",
     "CR": "urn:ietf:params:xml:ns:carddav",
     "D": "DAV:",
@@ -119,6 +130,11 @@ def _response(code):
     return "HTTP/1.1 %i %s" % (code, client.responses[code])
 
 
+def _href(href):
+    """Return prefixed href."""
+    return "%s%s" % (config.get("server", "base_prefix"), href.lstrip("/"))
+
+
 def name_from_path(path, collection):
     """Return Radicale item name from ``path``."""
     collection_parts = collection.path.strip("/").split("/")
@@ -143,13 +159,21 @@ def props_from_request(root, actions=("set", "remove")):
     prop_element = action_element.find(_tag("D", "prop"))
     if prop_element is not None:
         for prop in prop_element:
-            result[_tag_from_clark(prop.tag)] = prop.text
-            if prop.tag == "resourcetype":
+            if prop.tag == _tag("D", "resourcetype"):
                 for resource_type in prop:
-                    if resource_type.tag in ("calendar", "addressbook"):
-                        result["resourcetype"] = \
-                            "V%s" % resource_type.tag.upper()
+                    if resource_type.tag == _tag("C", "calendar"):
+                        result["tag"] = "VCALENDAR"
                         break
+                    elif resource_type.tag == _tag("CR", "addressbook"):
+                        result["tag"] = "VADDRESSBOOK"
+                        break
+            elif prop.tag == _tag("C", "supported-calendar-component-set"):
+                result[_tag_from_clark(prop.tag)] = ",".join(
+                    supported_comp.attrib["name"]
+                    for supported_comp in prop
+                    if supported_comp.tag == _tag("C", "comp"))
+            else:
+                result[_tag_from_clark(prop.tag)] = prop.text
 
     return result
 
@@ -174,7 +198,7 @@ def delete(path, collection):
     multistatus.append(response)
 
     href = ET.Element(_tag("D", "href"))
-    href.text = path
+    href.text = _href(path)
     response.append(href)
 
     status = ET.Element(_tag("D", "status"))
@@ -189,12 +213,23 @@ def propfind(path, xml_request, collections, user=None):
 
     Read rfc4918-9.1 for info.
 
+    The collections parameter is a list of collections that are
+    to be included in the output. Rights checking has to be done
+    by the caller.
+
     """
     # Reading request
-    root = ET.fromstring(xml_request.encode("utf8"))
-
-    prop_element = root.find(_tag("D", "prop"))
-    props = [prop.tag for prop in prop_element]
+    if xml_request:
+        root = ET.fromstring(xml_request.encode("utf8"))
+        props = [prop.tag for prop in root.find(_tag("D", "prop"))]
+    else:
+        props = [_tag("D", "getcontenttype"),
+                 _tag("D", "resourcetype"),
+                 _tag("D", "displayname"),
+                 _tag("D", "owner"),
+                 _tag("D", "getetag"),
+                 _tag("A", "calendar-color"),
+                 _tag("CS", "getctag")]
 
     # Writing answer
     multistatus = ET.Element(_tag("D", "multistatus"))
@@ -217,7 +252,7 @@ def _propfind_response(path, item, props, user):
 
     href = ET.Element(_tag("D", "href"))
     uri = item.url if is_collection else "%s/%s" % (path, item.name)
-    href.text = uri.replace("//", "/")
+    href.text = _href(uri.replace("//", "/"))
     response.append(href)
 
     propstat404 = ET.Element(_tag("D", "propstat"))
@@ -237,26 +272,35 @@ def _propfind_response(path, item, props, user):
             element.text = item.etag
         elif tag == _tag("D", "principal-URL"):
             tag = ET.Element(_tag("D", "href"))
-            tag.text = path
+            tag.text = _href(path)
             element.append(tag)
         elif tag in (_tag("D", "principal-collection-set"),
                      _tag("C", "calendar-user-address-set"),
                      _tag("CR", "addressbook-home-set"),
                      _tag("C", "calendar-home-set")):
             tag = ET.Element(_tag("D", "href"))
-            tag.text = path
+            tag.text = _href(path)
             element.append(tag)
         elif tag == _tag("C", "supported-calendar-component-set"):
             # This is not a Todo
             # pylint: disable=W0511
-            for component in ("VTODO", "VEVENT", "VJOURNAL"):
+            human_tag = _tag_from_clark(tag)
+            if is_collection and human_tag in collection_props:
+                # TODO: what do we have to do if it's not a collection?
+                components = collection_props[human_tag].split(",")
+            else:
+                components = ("VTODO", "VEVENT", "VJOURNAL")
+            for component in components:
                 comp = ET.Element(_tag("C", "comp"))
                 comp.set("name", component)
                 element.append(comp)
             # pylint: enable=W0511
         elif tag == _tag("D", "current-user-principal") and user:
             tag = ET.Element(_tag("D", "href"))
-            tag.text = "/%s/" % user
+            if item.resource_type == "addressbook":
+                tag.text = _href("/%s/addressbook.vcf/" % user)
+            else:
+                tag.text = _href("/%s/calendar.ics/" % user)
             element.append(tag)
         elif tag == _tag("D", "current-user-privilege-set"):
             privilege = ET.Element(_tag("D", "privilege"))
@@ -268,7 +312,7 @@ def _propfind_response(path, item, props, user):
             element.append(privilege)
         elif tag == _tag("D", "supported-report-set"):
             for report_name in (
-                    "principal-property-search", "sync-collection"
+                    "principal-property-search", "sync-collection",
                     "expand-property", "principal-search-property-set"):
                 supported = ET.Element(_tag("D", "supported-report"))
                 report_tag = ET.Element(_tag("D", "report"))
@@ -302,6 +346,8 @@ def _propfind_response(path, item, props, user):
                     item.tag, item.headers, item.timezones)
             elif tag == _tag("D", "displayname"):
                 element.text = item.name
+            elif tag == _tag("A", "calendar-color"):
+                element.text = item.color
             else:
                 human_tag = _tag_from_clark(tag)
                 if human_tag in collection_props:
@@ -380,7 +426,7 @@ def proppatch(path, xml_request, collection):
     multistatus.append(response)
 
     href = ET.Element(_tag("D", "href"))
-    href.text = path
+    href.text = _href(path)
     response.append(href)
 
     with collection.props as collection_props:
@@ -427,9 +473,12 @@ def report(path, xml_request, collection):
         if root.tag in (_tag("C", "calendar-multiget"),
                         _tag("CR", "addressbook-multiget")):
             # Read rfc4791-7.9 for info
-            hreferences = set(
-                href_element.text for href_element
-                in root.findall(_tag("D", "href")))
+            base_prefix = config.get("server", "base_prefix")
+            hreferences = set()
+            for href_element in root.findall(_tag("D", "href")):
+                href_path = unquote(urlparse(href_element.text).path)
+                if href_path.startswith(base_prefix):
+                    hreferences.add(href_path[len(base_prefix):])
         else:
             hreferences = (path,)
         # TODO: handle other filters
@@ -438,6 +487,7 @@ def report(path, xml_request, collection):
         tag_filters = set(
             element.get("name") for element
             in root.findall(".//%s" % _tag("C", "comp-filter")))
+        tag_filters.discard('VCALENDAR')
     else:
         hreferences = ()
         tag_filters = None
@@ -470,7 +520,7 @@ def report(path, xml_request, collection):
             multistatus.append(response)
 
             href = ET.Element(_tag("D", "href"))
-            href.text = "%s/%s" % (path.rstrip("/"), item.name)
+            href.text = _href("%s/%s" % (path.rstrip("/"), item.name))
             response.append(href)
 
             propstat = ET.Element(_tag("D", "propstat"))
@@ -483,6 +533,9 @@ def report(path, xml_request, collection):
                 element = ET.Element(tag)
                 if tag == _tag("D", "getetag"):
                     element.text = item.etag
+                elif tag == _tag("D", "getcontenttype"):
+                    element.text = "%s; component=%s" % (
+                        item.mimetype, item.tag.lower())
                 elif tag in (_tag("C", "calendar-data"),
                              _tag("CR", "address-data")):
                     if isinstance(item, ical.Component):
